/*
 ******************************* Creazione file *******************************
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago					Data di creazione : 30/03/2000
 * Codice   :
 *
 * Note:
 *
 *
 ******************************* Revisione file *******************************
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

#define VDW_MAIN

#define	MULTI_PCFOM	1

#include <vdw.h>
#include "function.h"
#include "NTPCFOMDriver.h"
#include "msglog.h"
#include "NTPCFOMDevice.h"


#pragma hdrstop("NTPCFOMDriver.pch")

// Set a default 32-bit tag value to be stored with each heap block
// allocated by operator new. Use BoundsChecker to view the memory pool.
// This value can be overridden using the global function SetPoolTag().
POOLTAG DefaultPoolTag('OFTN');

// Create the global driver trace object
// TODO:	Use KDebugOnlyTrace if you want trace messages
//			to appear only in debug builds.  Use KTrace if
//			you want trace messages to always appear.	
#if DBG
KTrace t("NTPCFOMDriver");
#endif

/////////////////////////////////////////////////////////////////////
// Begin INIT section
#pragma code_seg("INIT")

DECLARE_DRIVER_CLASS(NTPCFOMDriver, L"LOADED NTPFCOM DRIVER RESOURCES")

#ifdef	MULTI_PCFOM

NTSTATUS RegCfgCallback( ULONG unit, PVOID context);

/////////////////////////////////////////////////////////////////////////////
// RegCfgCallback
/////////////////////////////////////////////////////////////////////////////

NTSTATUS RegCfgCallback( ULONG Unit, PVOID Context)
{
	NTSTATUS	status	=	STATUS_SUCCESS;
	
	KUnitizedName* pRegPath = KDevice::CreateRegistryPath(L"NTPCFOMDevice", Unit);

	KRegistryKey UnitKey(*pRegPath, (PCWSTR)0);

   // UnitKey is the open registry key for the device unit
   // call device class constructor
	
	NTPCFOMDevice*	p_NTPCFOMDevice = new (
			static_cast<PCWSTR>(KUnitizedName(L"NTPCFOMDevice", Unit)),
			FILE_DEVICE_UNKNOWN,
			static_cast<PCWSTR>(KUnitizedName(L"NTPCFOMDevice", Unit)), 
			0,
			DO_BUFFERED_IO
			)
		NTPCFOMDevice(Unit);

	if (p_NTPCFOMDevice	== NULL)
	{
#if DBG
		t << "Error constructing device NTPCFOMDevice" << EOL;
#endif
		return STATUS_INSUFFICIENT_RESOURCES;
	}
	
	status = p_NTPCFOMDevice->ConstructorStatus();
	
	if (!NT_SUCCESS(status))
	{
		// Error returned from a constructor
#if DBG
		t << "Error creating device NTPCFOMDevice, status " << (ULONG) status << EOL;
#endif
		delete p_NTPCFOMDevice;
		return status;
	}
	   
   *(NTSTATUS*)Context = status;

   return	status;
}

#endif

/////////////////////////////////////////////////////////////////////
//  NTPCFOMDriver::DriverEntry
// 
//	Routine Description:
//		This routine is called when the driver is loaded.
//
//	Parameters:
//		RegistryPath - String used to find driver parameters in the
//			registry.  To locate NTPCFOMDriver look for:
//			HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\NTPCFOMDriver
//
//	Return Value:
//		NTSTATUS - Return STATUS_SUCCESS if no errors are encountered.
//			Any other indicates to the system that an error has occured.
//
//	Comments:
//		Usually, this is where any devices associated with
//		the driver are created.
//
//		The driver often reads the registry at DriverEntry in
//		order to setup various configurable parameters.
//		
//		DriverWorks makes it easy to use the registry to also
//		control what devices are present and should be created.
//

NTSTATUS NTPCFOMDriver::DriverEntry(PUNICODE_STRING RegistryPath)
{
	NTSTATUS status;			// Status of device creation
	
	BOOLEAN DeviceConstructed = FALSE;	

#if DBG
	t << "In DriverEntry Compiled at " __TIME__ " on " __DATE__ "\n";
#endif
	// Open the "Parameters" key under the driver
	KRegistryKey Params(RegistryPath, L"Parameters");

	if ( NT_SUCCESS(Params.LastError()) )
	{
#if DBG
		ULONG bBreakOnEntry = FALSE;
			// Read "BreakOnEntry" value from registry
		Params.QueryValue(L"BreakOnEntry", &bBreakOnEntry);
			// If requested, break into debugger
		if (bBreakOnEntry) DbgBreakPoint();

#endif
		// Load driver data members from the registry
		LoadRegistryParameters(Params);
	}

#ifdef MULTI_PCFOM

	KConfigurationQuery AtoDQuery( RegCfgCallback, &status, L"NTPCFOMDevice");

#else

	int	Unit = 0;

	// Create NTPCCIODevice. Note that we used a form of "placement" new,
	// that is a member operator of KDevice.  This form will use storage
	// allocated by the system in the device object's device to store our
	// class instance.
	NTPCFOMDevice*	p_NTPCFOMDevice = new (
			static_cast<PCWSTR>(KUnitizedName(L"NTPCFOMDevice", Unit)),
			FILE_DEVICE_UNKNOWN,
			static_cast<PCWSTR>(KUnitizedName(L"NTPCFOMDevice", Unit)), 
			0,
			DO_BUFFERED_IO
			)
		NTPCFOMDevice();

	if (p_NTPCFOMDevice	== NULL)
	{
#if DBG
		t << "Error constructing device NTPCFOMDevice" << EOL;
#endif
		return STATUS_INSUFFICIENT_RESOURCES;
	}
	
	status = p_NTPCFOMDevice->ConstructorStatus();
	
	if (!NT_SUCCESS(status))
	{
		// Error returned from a constructor
#if DBG
		t << "Error creating device NTPCFOMDevice, status " << (ULONG) status << EOL;
#endif
		delete p_NTPCFOMDevice;
		return status;
	}

#endif

	// If no errors returned during device construction, return success code.
	return STATUS_SUCCESS;
}

/////////////////////////////////////////////////////////////////////
//  NTPCFOMDriver::LoadRegistryParameters
//
//	Routine Description:
//		Load driver data members from the registry.
// 
//	Parameters:
//		Params - Open registry key pointing to "Parameters"
//
//	Return Value:
//		None
//
//	Comments:
//		The parameters are found as values under the "Parameters" key,	
//		HKLM\SYSTEM\CurrentControlSet\Services\NTPCFOMDriver\Parameters\...
//			

void NTPCFOMDriver::LoadRegistryParameters(KRegistryKey &Params)
{

	m_bBreakOnEntry = FALSE;
	Params.QueryValue(L"BreakOnEntry", &m_bBreakOnEntry);
#if DBG
	t << "m_bBreakOnEntry loaded from registry, resulting value: [" << m_bBreakOnEntry << "]\n";
#endif
}



#pragma code_seg()

/////////////////////////////////////////////////////////////////////
//  NTPCFOMDriver::Unload
// 
//	Routine Description:
//		This routine is called when the driver is unloaded.
//
//	Parameters:
//		None
//
//	Return Value:
//		None
//
//	Comments:
//		Unload is responsible for releasing any system objects that
//		the driver has allocated. 
//
//		In general, this function must comprehensively ensure that
//		the driver is not unloaded while holding system objects,
//		including memory, or while there are pending events that
//		would cause the system to call the driver. This is best done
//		by deconstructing top level objects, which in turn release
//		objects for which they are responsible.
//
//		This function is called at PASSIVE_LEVEL.
//

VOID NTPCFOMDriver::Unload(VOID)
{
#if DBG
	t << "Unload called\n";
#endif
	// If you don't need to perform any functions
   // except to call the base class KDriver::Unload(),
   // then this entire routine may be safely deleted.

    // Call base class to delete all devices.
	KDriver::Unload();
}
